﻿use strict;
use Encode ();
#------------------------------------------------------------------------------
# 郵便番号記法プラグイン
#                                                   (C)2012 nabe / nabe@abk.nu
#------------------------------------------------------------------------------
package Satsuki::TextParser::TagPlugin::Tag_zipcode;
###############################################################################
# ■基本処理
###############################################################################
#------------------------------------------------------------------------------
# ●コンストラクタ
#------------------------------------------------------------------------------
sub new {
	my $class = shift;	# 読み捨て
	my $ROBJ  = shift;	# 読み捨て
	my $tags  = shift;
	$tags->{"zipcode"} = {data => \&zipcode};
	$tags->{"zippref"} = {data => \&zipcode};
}

###############################################################################
# ■タグ処理ルーチン
###############################################################################
#------------------------------------------------------------------------------
# ●zipcode記法
#------------------------------------------------------------------------------
sub zipcode {
	my ($parser_obj, $tag, $cmd, $ary) = @_;
	my $ROBJ = $parser_obj->{ROBJ};

	my $code = shift(@$ary);	# 123-4567
	$code =~ s/-//g;
	my $c3 = substr($code, 0, 3);	# 123
	my $c4 = substr($code, 3, 4);	# 4567

	my $h = &load_zipcode_data($ROBJ, $c3);
	if (!$h) { return ''; }		# error

	my $ret = ($cmd eq 'zippref') ? $h->{pref} : $h->{pref} . $h->{$c4};
	$ret = $h->{$c4} ? "<span title=\"〒$c3-$c4\">$ret</span>" : $ret;
	# ↑$h->{$c4}が存在しないときはspanを入れない。

	# code変換
	my $jcode = $ROBJ->loadpm('Jcode');
	$jcode->from_to(\$ret, 'UTF-8', $ROBJ->{System_coding});
	return $ret;
}

###############################################################################
# ■サブルーチン
###############################################################################
#------------------------------------------------------------------------------
# ●データロード
#------------------------------------------------------------------------------
sub load_zipcode_data {
	my ($ROBJ, $code) = @_;
	my $dir = $ROBJ->{Diary}->{public_dir} . 'zipcode/';

	if ($code !~ /^\d\d\d$/) { return ; } 
	my $lines = $ROBJ->fread_lines_cached("$dir$code.dat");

	my $pref = shift(@$lines);
	chomp($pref);

	my %h = ('pref' => $pref);
	my $prev;
	foreach(@$lines) {
		if ($_ !~ /^(\d+)(.*?)(\d*)$/) { next; }
		my $adr = $2;

		Encode::_utf8_on($adr);
		if ($3) {
			$adr = substr($prev, 0, $3) . $adr;
		}
		$prev = $adr;
		Encode::_utf8_off($adr);

		$h{ sprintf("%04d", $1) } = $adr;
	}
	return \%h;
}

1;
